package com.css.simulation.resource.scheduler.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.dto.ProjectMessageDTO;
import api.common.util.*;
import com.css.simulation.resource.scheduler.mapper.*;
import com.css.simulation.resource.scheduler.pojo.po.*;
import com.css.simulation.resource.scheduler.pojo.to.*;
import com.css.simulation.resource.scheduler.util.GitUtil;
import com.css.simulation.resource.scheduler.util.KubernetesUtil;
import com.css.simulation.resource.scheduler.util.MinioUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.kubernetes.client.openapi.ApiClient;
import io.kubernetes.client.openapi.models.V1Pod;
import io.kubernetes.client.util.Yaml;
import io.minio.MinioClient;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ProjectService {

    @Value("${scheduler.minio-path.project-result}")
    String projectResultPathOfMinio;
    @Value("${scheduler.linux-path.temp}")
    String linuxTempPath;
    @Value("${scheduler.linux-path.pod-template-yaml}")
    String podTemplateYaml;
    @Value("${scheduler.linux-path.pod-yaml-directory}")
    String podYamlDirectory;
    @Value("${minio.bucket-name}")
    String bucketName;
    @Value("${scheduler.host.hostname}")
    String hostname;
    @Value("${scheduler.host.username}")
    String username;
    @Value("${scheduler.host.password}")
    String password;
    @Value("${scheduler.kubernetes.namespace}")
    String kubernetesNamespace;
    @Resource
    StringRedisTemplate stringRedisTemplate;
    @Resource
    KafkaTemplate<String, String> kafkaTemplate;
    @Resource
    CloseableHttpClient closeableHttpClient;
    @Resource
    RequestConfig requestConfig;
    @Resource
    MinioClient minioClient;
    @Resource
    TaskMapper taskMapper;
    @Resource
    IndexTemplateMapper indexTemplateMapper;
    @Resource
    IndexMapper indexMapper;
    @Resource
    SceneMapper sceneMapper;
    @Resource
    AlgorithmMapper algorithmMapper;
    @Resource
    ApiClient apiClient;


    // -------------------------------- Comment --------------------------------

    /**
     * @param nodeMap           节点列表以及剩余可用并行度
     * @param projectMessageDTO 初始接收到的项目启动信息
     * @param clusterPrefix     clusterPrefix
     */
    @Transactional
    public void prepare(Map<String, Integer> nodeMap, ProjectMessageDTO projectMessageDTO, String clusterPrefix) {
        String projectId = projectMessageDTO.getProjectId();
        String projectType = projectMessageDTO.getType();
        //1 将指定 node 的并行度减少
        nodeMap.keySet().forEach(nodeName -> {
            int parallelismToUse = nodeMap.get(nodeName);
            String restParallelismKey = "node:" + nodeName + ":parallelism";
            int restParallelism = Integer.parseInt(Objects.requireNonNull(stringRedisTemplate.opsForValue().get(restParallelismKey)));// 剩余可用并行度
            stringRedisTemplate.opsForValue().set(restParallelismKey, (restParallelism - parallelismToUse) + "");
        });
        //2 将 redis 中该项目旧的信息则直接删除（包括 waitingKey)
        Set<String> oldKeys = stringRedisTemplate.keys(clusterPrefix + "*");
        if (CollectionUtil.isNotEmpty(oldKeys)) {
            for (String oldKey : oldKeys) {
                if (oldKey.contains(projectId)) {
                    stringRedisTemplate.delete(oldKeys);
                }
            }
        }

//        //4 将项目状态修改为执行中（页面上已经改成了执行中，这里应该不需要）
//        if (DictConstants.PROJECT_TYPE_MANUAL.equals(projectType)) {
//            manualProjectMapper.updateNowRunStateById(DictConstants.PROJECT_RUNNING, projectId);
//        } else if (DictConstants.PROJECT_TYPE_AUTO_SUB.equals(projectType)) {
//            autoSubProjectMapper.updateNowRunStateById(DictConstants.PROJECT_RUNNING, projectId);
//        }

        //5 将该 project 下所有旧的任务和指标得分删除。
        taskMapper.deleteByProject(projectId);
        indexMapper.deleteFirstTargetScoreByProjectId(projectId);
        indexMapper.deleteLastTargetScoreByProjectId(projectId);
    }

    @SneakyThrows
    @Transactional
    public List<ScenePO> handlePackage(String projectRunningKey, String projectId, String packageId) {
        String allIndexPrefix = projectRunningKey + ":package:" + packageId + ":all";
        String leafIndexPrefix = projectRunningKey + ":package:" + packageId + ":leaf";

        //1 查询该场景包的所有指标列表存入 redis，包删了无所谓，但要过滤删掉的指标
        List<IndexTemplatePO> allIndexList = indexTemplateMapper.selectByPackageIdIncludeDeleted(packageId);
        stringRedisTemplate.opsForValue().set(allIndexPrefix, JsonUtil.listToJson(allIndexList));

        //2 查询场景包叶子指标
        List<IndexTemplatePO> leafIndexList = allIndexList.stream().filter(index -> StringUtil.isNotEmpty(index.getRuleId())).collect(Collectors.toList());
        stringRedisTemplate.opsForValue().set(leafIndexPrefix, JsonUtil.listToJson(leafIndexList));
        log.info("ProjectService--handlePackage 项目 " + projectId + " 的叶子指标为：" + leafIndexList);
        List<ScenePO> sceneList = new ArrayList<>();
        leafIndexList.forEach(leafIndex -> {
            String naturalIds = leafIndex.getSceneNaturalIds();
            String standardIds = leafIndex.getSceneStatueIds();
            String accidentIds = leafIndex.getSceneTrafficIds();
            String generalizationIds = leafIndex.getSceneGeneralizationIds();
            if (StringUtil.isNotEmpty(naturalIds)) {
                List<String> naturalIdList = new ArrayList<>(Arrays.asList(naturalIds.split(",")));
                sceneList.addAll(sceneMapper.selectNaturalByIdList(naturalIdList));
            }
            if (StringUtil.isNotEmpty(standardIds)) {
                List<String> standardIdList = new ArrayList<>(Arrays.asList(standardIds.split(",")));
                sceneList.addAll(sceneMapper.selectStandardByIdList(standardIdList));
            }
            if (StringUtil.isNotEmpty(accidentIds)) {
                List<String> accidentIdList = new ArrayList<>(Arrays.asList(accidentIds.split(",")));
                sceneList.addAll(sceneMapper.selectAccidentByIdList(accidentIdList));
            }
            if (StringUtil.isNotEmpty(generalizationIds)) {
                List<String> generalizationIdList = new ArrayList<>(Arrays.asList(generalizationIds.split(",")));
                sceneList.addAll(sceneMapper.selectGeneralizationByIdList(generalizationIdList));
            }
        });
        log.info("ProjectService--handlePackage 项目" + projectId + " 共有 " + sceneList.size() + " 个任务！");
        return sceneList;
    }


    /**
     * @param projectRunningPrefix
     * @param userId
     * @param projectId
     * @param videoTime            视频长度
     * @param scenePOSet
     * @param vehiclePO
     * @param cameraPOList
     * @param ogtPOList
     */
    public void sendTaskMessage(String projectRunningPrefix, String userId, String projectId, String projectType, Long videoTime, Set<ScenePO> scenePOSet, VehiclePO vehiclePO, List<CameraPO> cameraPOList, List<OgtPO> ogtPOList) {
        final int[] messageNumber = {0};
        log.info("ProjectService--sendTaskMessage 项目 " + projectId + " 获得的包括的场景信息为：" + scenePOSet);
        for (ScenePO scenePO : scenePOSet) {
            String sceneId = scenePO.getId();
            //3-1 可能会存在多个指标下有同样的场景，所以会查出多个指标，多个指标的场景需要发送多次
            List<String> lastTargetIdList = null;
            if (DictConstants.PROJECT_TYPE_MANUAL.equals(projectType)) {
                lastTargetIdList = indexMapper.selectLeafIndexIdByManualProjectIdAndSceneId(projectId, "%" + sceneId + "%");
            } else if (DictConstants.PROJECT_TYPE_AUTO_SUB.equals(projectType)) {
                lastTargetIdList = indexMapper.selectLeafIndexIdByAutoSubProjectIdAndSceneId(projectId, "%" + sceneId + "%");
            }
            lastTargetIdList.forEach(lastTargetId -> {
                String taskId = StringUtil.getRandomUUID();
                String taskRetryPrefix = projectRunningPrefix + ":task:" + taskId + ":retry";
                String taskMessagePrefix = projectRunningPrefix + ":task:" + taskId + ":message";
                // 设置任务重试次数为 0，方便任务进行最大3次的重试。
                stringRedisTemplate.opsForValue().set(taskRetryPrefix, "0");
                // 保存任务信息
                TaskPO taskPO = TaskPO.builder() // run_start_time 和 run_end_time 不填
                        .id(taskId)
                        .pId(projectId)
                        .sceneId(sceneId)
                        .lastTargetId(lastTargetId)
                        .sceneName(scenePO.getName())
                        .sceneType(scenePO.getType())
                        .runState(DictConstants.TASK_PENDING)
                        .runResultFilePath(projectResultPathOfMinio + projectId + "/" + taskId)
                        .build();
                taskPO.setCreateTime(TimeUtil.getNowForMysql());
                taskPO.setCreateUserId(userId);
                taskPO.setModifyTime(TimeUtil.getNowForMysql());
                taskPO.setModifyUserId(userId);
                taskPO.setModifyTime(TimeUtil.getNowForMysql());
                taskPO.setIsDeleted("0");
                taskMapper.insert(taskPO);

                // 组装 task 消息
                TaskTO taskTO = TaskTO.builder()
                        .info(InfoTO.builder()
                                .project_id(taskPO.getPId())
                                .task_id(taskPO.getId())
                                .task_path(taskPO.getRunResultFilePath())
                                .default_time(videoTime)
                                .build())
                        .scenario(ScenarioTO.builder()
                                .scenario_osc(scenePO.getScenarioOsc())
                                .scenario_odr(scenePO.getScenarioOdr())
                                .scenario_osgb(scenePO.getScenarioOsgb())
                                .build())
                        .vehicle(VehicleTO.builder()
                                .model(ModelTO.builder()
                                        .model_label(vehiclePO.getModelLabel())
                                        .build())
                                .dynamics(DynamicsTO.builder()
                                        .dynamics_maxspeed(vehiclePO.getMaxSpeed())
                                        .dynamics_enginepower(vehiclePO.getEnginePower())
                                        .dynamics_maxdecel(vehiclePO.getMaxDeceleration())
                                        .dynamics_maxsteering(vehiclePO.getMaxSteeringAngle())
                                        .dynamics_mass(vehiclePO.getMass())
                                        .dynamics_frontsurfaceeffective(vehiclePO.getFrontSurfaceEffective())
                                        .dynamics_airdragcoefficient(vehiclePO.getAirDragCoefficient())
                                        .dynamics_rollingresistance(vehiclePO.getRollingResistanceCoefficient())
                                        .dynamics_wheeldiameter(vehiclePO.getWheelDiameter())
                                        .dynamics_wheeldrive(vehiclePO.getWheelDrive())
                                        .dynamics_overallefficiency(vehiclePO.getOverallEfficiency())
                                        .dynamics_distfront(vehiclePO.getFrontDistance())
                                        .dynamics_distrear(vehiclePO.getRearDistance())
                                        .dynamics_distleft(vehiclePO.getLeftDistance())
                                        .dynamics_distright(vehiclePO.getRightDistance())
                                        .dynamics_distheight(vehiclePO.getHeightDistance())
                                        .dynamics_wheelbase(vehiclePO.getWheelbase())
                                        .build())
                                .sensors(SensorsTO.builder()   // 根据 vehicleId 查询绑定的传感器列表
                                        .camera(cameraPOList)
                                        .OGT(ogtPOList)
                                        .build())
                                .build())
                        .build();

                //4-4 将对象转成 json
                String taskJson = "";
                try {
                    taskJson = JsonUtil.beanToJson(taskTO);
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
                //4-5 将 projectId 作为 topic 名称，发送 task 信息到 kafka
                String finalTaskJson = taskJson;
                stringRedisTemplate.opsForValue().set(taskMessagePrefix, finalTaskJson);
                kafkaTemplate.send(projectId, taskJson).addCallback(success -> {
                    // 消息发送到的topic
                    String topic = success.getRecordMetadata().topic();
                    // 消息发送到的分区
                    int partition = success.getRecordMetadata().partition();
                    // 消息在分区内的offset
                    long offset = success.getRecordMetadata().offset();
                    log.info("------- ProjectConsumer 发送消息成功：\n"
                            + "主题 topic 为：" + topic + "\n"
                            + "分区 partition 为：" + partition + "\n"
                            + "偏移量为：" + offset + "\n"
                            + "消息体为：" + finalTaskJson);
                }, failure -> log.error("------- 发送消息失败：" + failure.getMessage()));
                messageNumber[0] = messageNumber[0] + 1;
            });
        }
        log.info("ProjectService--sendTaskMessage 共发送了 " + messageNumber[0] + " 条消息！");
    }


    /**
     * 将 master 节点设置成镜像仓库，导入镜像的同时 commit 到仓库当中，供其他节点 pull
     *
     * @param projectId
     * @param algorithmId
     * @return
     */
    @SneakyThrows
    @Transactional
    public String handleAlgorithm(String projectId, String algorithmId) {
        //4-1 根据算法 id 获取算法文件地址、是否已导入成镜像。
        AlgorithmPO algorithmPO = algorithmMapper.selectById(algorithmId);
        String dockerImage;
        String algorithmTarLinuxTempPath;

        if (algorithmPO == null) {
            dockerImage = "algorithm_" + algorithmId + ":latest";
            algorithmTarLinuxTempPath = linuxTempPath + "algorithm/" + algorithmId + ".tar";
            // 访问索为远程接口
            String tokenUri = "http://open-api.zoogooy.com/cgi-bin/token/token?grant_type=client_credential";
            String appid = "3e64be4a29e5478f9717d53c11ab26ad";
            String secret = "f183079f97ac9ed81a864619a83fc17a";
            String tokenUrl = tokenUri + "&appid=" + appid + "&secret=" + secret;
            String result = HttpUtil.get(closeableHttpClient, requestConfig, tokenUrl);
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode jsonNode = objectMapper.readTree(result);
            JsonNode dataNode = jsonNode.path("data");
            String token = dataNode.path("access_token").asText();

            /*
            {
              "data" : {
                "totalElements" : 19,
                "totalPage" : null,
                "currentPage" : 0,
                "size" : 0,
                "content" : [ {
                  "id" : "ICV20220418170905",
                  "algorithmId" : "2022041813435600064848bbc0e0efb441978d57",
                  "algorithmName" : "SUAN FA2",
                  "description" : "<div class=\"index-module_textWrap_3ygOc \">\n<p>4月17日，中科院图书馆处某位负责落实此事的老师向红星新闻证实，网传的近“千万级别”续订费用和停用知网访问一事属实，中科院各大所正积极应对因停订知网带来的影响。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>“自2022年4月20日起，停用CNKI数据库。以后会用万方和维普替代，查不到的文献可以用文献传递等方式。”至于何时会恢复对知网的访问，该老师表示，目前并不确定，“至少今年应该没有这个计划了”。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>去年底，89岁退休教授赵德馨起诉中国知网，引发热议。原因是后者擅自收录他的100多篇论文，赵德馨没拿到一分钱稿费，自己下载还要付费。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>最终赵德馨全部胜诉，累计获赔70余万元。中国知网不再收录他的文章，已收录的也全部下架。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>据中新经纬2021年报道，博士论文、硕士论文在中国知网出版，作者本人最高仅可获得100元现金，以及400元面值的检索阅读卡作为稿酬。但是作者的论文每在中国知网上被下载一次，平台就会收取15元/本甚至25元/本的费用。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>在这种模式下，中国知网吸金不少。财报数据显示，2020年中国知网年收入近12亿元，毛利率近54%。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>天眼查信息显示，中国知网所属公司为同方知网(北京)技术有限公司(下称同方知网)，该公司成立于2004年11月18日，是A股上市公司同方股份的子公司。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>目前，中国知网已经建成了世界上全文信息量规模最大的“CNKI数字图书馆”，成为中国高校师生最常用的文献资料数据库。</p>\n<p>&nbsp;</p>\n</div>\n<div class=\"index-module_textWrap_3ygOc \">\n<p>&nbsp;</p>\n<p>据同方股份年报，同方知网主要从事互联网出版与服务业务，目前已经形成“中国知网”(CNKI)门户网站，为用户提供《中国知识资源总库》《中国学术期刊数据库》《中国博硕士论文数据库》、《中国年鉴全文数据库》《中国工具书网络出版总库》等一系列产品，以及知识资源互联网搜索、共享和网络出版服务。同方股份财报显示，2020年全年，同方知网主营业务收入11.68亿元、归母净利润1.93亿元，毛利率53.93%；2021年上半年，该公司主营业务收入4.96亿元、归母净利润1892.70万元，毛利率为51.30%。据悉，中国知网曾因频繁涨价遭到高校抵制。2016年1月，武汉理工大学发布了知网停用的通知。校方称：“由于续订价格涨价离谱，我校与中国知网公司的谈判不成功。这些年来，CNKI公司涨价幅度过大的行为已经收到全国很多高校的抵制，包括许多知名的985高校。”武汉理工大学图书馆称，2000年以来，知网每年的报价涨幅都超过10%，从2010年到2016年的报价涨幅为132.86%，年平均涨幅为18.98%。但在不到1个月后，武汉理工大学又重新订购并恢复开通中国知网数据。</p>\n</div>",
                  "algorithmType" : "2"
                }, {
                  "id" : "ICV20220408165152",
                  "algorithmId" : "202204081650140005916540f84c37764591a5c4",
                  "algorithmName" : "ceshi 33",
                  "description" : "",
                  "algorithmType" : "2"
                } ],
                "hasPreviousPage" : false,
                "hasNextPage" : false
              },
              "success" : true,
              "message" : "ok",
              "code" : 1,
              "nowTime" : "2022-04-22 10:14:40"
            }
             */
            String downloadUrl = "http://open-api.zoogooy.com/cgi-bin/api/icv-algorithm-agg/simulation/download"
                    + "?access_token=" + token
                    + "&id=" + algorithmId;
            String tempDownloadUrl = HttpUtil.get(closeableHttpClient, requestConfig, downloadUrl);
            InputStream inputStream = HttpUtil.getInputStream(closeableHttpClient, requestConfig, tempDownloadUrl);
            FileUtil.writeInputStreamToLocalFile(inputStream, algorithmTarLinuxTempPath);
            LinuxUtil.execute("docker import " + algorithmTarLinuxTempPath + " " + dockerImage);
        } else {
            String algorithmCode = algorithmPO.getAlgorithmCode();
            dockerImage = "algorithm_" + algorithmCode + ":latest";
            algorithmTarLinuxTempPath = linuxTempPath + "algorithm/" + algorithmCode + ".tar";
            if ("1".equals(algorithmPO.getDockerImport()) && StringUtil.isNotEmpty(algorithmPO.getDockerImage())) {
                dockerImage = algorithmPO.getDockerImage();
            } else if ("0".equals(algorithmPO.getDockerImport()) || algorithmPO.getDockerImport() == null) {
                String uploadMode = algorithmPO.getUploadMode();
                if (DictConstants.ALGORITHM_UPLOAD_MODE_FILE.equals(uploadMode)) {
                    String minioPath = algorithmPO.getMinioPath();
                    MinioUtil.downloadToFile(minioClient, bucketName, minioPath, algorithmTarLinuxTempPath);    // 下载算法文件到本地
                } else if (DictConstants.ALGORITHM_UPLOAD_MODE_GIT.equals(uploadMode)) {
                    String gitUrl = algorithmPO.getGitUrl();
                    String gitUserName = algorithmPO.getGitUserName();
                    String gitPassword = algorithmPO.getGitPassword();
                    GitUtil.clone(gitUrl, gitUserName, gitPassword, algorithmTarLinuxTempPath); // 下载算法文件到本地
                } else {
                    throw new RuntimeException("算法 " + algorithmId + " 的 upload_mode 字段数据有误！");
                }
                // 本地执行 docker load 算法文件成镜像（可改成用 docker-java 操作仓库）
                LinuxUtil.execute("docker import " + algorithmTarLinuxTempPath + " " + dockerImage);
                algorithmMapper.updateDockerImportAndDockerImageById("1", dockerImage, algorithmId);
                FileUtil.rm(algorithmTarLinuxTempPath);
                log.info("ProjectService--handleAlgorithm 已删除算法临时文件：" + algorithmTarLinuxTempPath);
            } else {
                throw new RuntimeException("算法 " + algorithmId + " 的 mysql 数据有误！");
            }
        }
        log.info("ProjectService--handleAlgorithm 项目 " + projectId + " 使用的算法镜像为：" + dockerImage);
        return dockerImage;
    }

//    /**
//     * 运行
//     *
//     * @param projectId            项目id
//     * @param nodeMap              并行度
//     * @param algorithmDockerImage 算法镜像
//     */
//    @SneakyThrows
//    public void transferAndRunYaml(String projectId, Map<String, Integer> nodeMap, String algorithmDockerImage) {
//        String podTemplate = FileUtil.read(podTemplateYaml);
//        String replace0 = podTemplate.replace("vtd-container", "vtd-" + projectId);
//        String replace1 = replace0.replace("algorithm-container", "algorithm-" + projectId);
//        String replace2 = replace1.replace("algorithm-image", algorithmDockerImage);
//        String replace3 = replace2.replace("kafkaTopic", projectId);     // 消息主题名称为 projectId
//        nodeMap.forEach((nodeName, parallelism) -> {
//            String tempPodNameSuffix = projectId + StringUtil.getRandomUUID();
//            String tempReplace4 = replace3.replace("pod-name", "project-" + tempPodNameSuffix); // pod 名称包括 projectId 和 随机字符串
//            String tempFinalYaml = tempReplace4.replace("node-name", nodeName);     // 指定 pod 运行节点
//            log.info("ProjectConsumer--parseManualProject 在节点 " + nodeName + " 开始执行 pod：" + finalYaml);
//            String tempFinalYamlTargetPath = podYamlDirectory + tempPodNameSuffix;
//            FileUtil.writeStringToLocalFile(tempFinalYaml, tempFinalYamlTargetPath);
//            //  启动
//            KubernetesUtil.applyYaml(hostname, username, password, jobTemplateYamlPathTarget);
//        });
//    }

    /**
     * 运行 pod
     *
     * @param projectId            项目id
     * @param nodeMap              并行度
     * @param algorithmDockerImage 算法镜像
     */
    @SneakyThrows
    public void createPod(String projectId, Map<String, Integer> nodeMap, String algorithmDockerImage) {
        String podString = FileUtil.read(new File(podTemplateYaml));
        String replace0 = podString.replace("vtd-container", "vtd-" + projectId);
        String replace1 = replace0.replace("algorithm-container", "algorithm-" + projectId);
        String replace2 = replace1.replace("algorithm-image", algorithmDockerImage);
        String podTemplateStringOfProject = replace2.replace("kafkaTopic", projectId);     // 消息主题名称为 projectId
        String podTemplateFileNameOfProject = "project-" + projectId + ".yaml";     // 模板文件名称
        FileUtil.writeStringToLocalFile(podTemplateStringOfProject, podYamlDirectory + podTemplateFileNameOfProject);

        nodeMap.forEach((nodeName, parallelism) -> {
            String podName = "project-" + projectId + "-" + StringUtil.getRandomUUID();
            stringRedisTemplate.opsForValue().set("pod:" + podName + ":node", nodeName);    // 将 pod 运行在哪个 node 上记录到 redis
            String tempPodFileNameOfProject = podName + ".yaml";     // 模板文件名称
            // -------------------------------- Comment --------------------------------
            String tempReplace4 = podTemplateStringOfProject.replace("pod-name", podName); // pod 名称包括 projectId 和 随机字符串
            String tempPodString = tempReplace4.replace("node-name", nodeName);     // 指定 pod 运行节点
            log.info("ProjectConsumer--parseManualProject 在节点 " + nodeName + " 开始执行 pod：" + tempPodString);
            V1Pod v1Pod;
            try {
                FileUtil.writeStringToLocalFile(tempPodString, podYamlDirectory + tempPodFileNameOfProject);
                v1Pod = (V1Pod) Yaml.load(tempPodString);
            } catch (IOException e) {
                throw new RuntimeException("ProjectService--createPod 创建 pod 失败，项目为：" + projectId);
            }
            //  启动
            KubernetesUtil.createPod(apiClient, kubernetesNamespace, v1Pod);
        });
    }


    @SneakyThrows
    public void stopProject(String projectId, String projectType) {
        //2 根据 pod 前缀删除所有 pod
        String prefix = "project-" + projectId;
        KubernetesUtil.deleteJob(apiClient, kubernetesNamespace, prefix);

        //3 删除所有 redis key
//        PrefixTO redisPrefix = projectUtil.getRedisPrefixByProjectIdAndProjectType(projectId, type);
//        Set<String> keys = stringRedisTemplate.keys(redisPrefix.getProjectRunningKey() + "*");
//        if (CollectionUtil.isNotEmpty(keys)) {
//            for (String key : keys) {
//                stringRedisTemplate.delete(key);
//            }
//        }
    }


}
